%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BENEFITS OF SHARING INFORMATION IN ITERATIVE LEARNING CONTROL
%   - The covariance matrix P -
%
%   Angela Schllig, Oct 2009 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

n = 5;
syms a b 

H = [zeros(n,1) eye(n)]

% initial covariance Po
% !!ATTENTION!!
% - index j is shifted by 1 compared to the representation in the paper
P{1} = a*ones(n+1,n+1) + diag([0,b*ones(1,n)])

ITER = 3;

% calculate it
for k = 2:ITER+1
    k
    Q{k} = simplify(H*P{k-1}*H' + eye(n))
    simplify(P{k-1}*H')
    simplify(inv(Q{k}))
    K{k} = simplify(P{k-1}*H'*inv(Q{k}))
    P{k} = simplify( (eye(n+1) - K{k}*H)*P{k-1})
end
 

% proof
clearvars n
syms p00 p01 p11 p12 p00_j p01_j p11_j p12_j n 

m11 = (1+p11+(n-2)*p12)/(p11+1-p12)/(p11+1+(n-1)*p12)
m12 = (-p12)/(p11+1-p12)/(p11+1+(n-1)*p12)

k01 = simplify(p01 * (m11 + (n-1)*m12))
k11 = simplify(p11 * m11 + (n-1) *p12 *m12)
k12 = simplify(p12 *m11 + p11*m12 + (n-2) *p12 *m12)

% gives:
% k01 = p01/(1+p11+p12*n-p12)
% k11 = (p11+p11^2+p11*p12*n-2*p11*p12-p12^2*n+p12^2)/(1+p11-p12)/(1+p11+p12*n-p12)
% k12 = p12/(1+p11-p12)/(1+p11+p12*n-p12)

p00_j = simplify(p00 -n*p01*k01)
p01_j = simplify(p01-k01*p11-(n-1)*k01*p12)
p10_j = simplify((1-k11)*p01-(n-1)*k12*p01)
p11_j = simplify((1-k11)*p11-(n-1)*k12*p12)
p12_j = simplify((1-k11)*p12-k12*p11-(n-2)*k12*p12)

% gives:
% p00_j = -(-p00-p00*p11-p00*p12*n+p00*p12+n*p01^2)/(1+p11+p12*n-p12)
% p01_j = p10_n = p01/(1+p11+p12*n-p12)
% p11_j = (p11+p11^2+p11*p12*n-2*p11*p12-p12^2*n+p12^2)/(1+p11-p12)/(1+p11+p12*n-p12)
% p12_j = p12/(1+p11-p12)/(1+p11+p12*n-p12)

syms j

disp('Expected Result')
p00 = a*(1+(j-1)*b)/(1+(j-1)*b+(j-1)*n*a)
p01 = a/(1+(j-1)*b+(j-1)*n*a)
p11 = (a+b+(j-1)*b*a*n+(j-1)*b*b)/(1+(j-1)*b+(j-1)*n*a)/(1+(j-1)*b)
p12 = a/(1+(j-1)*b+(j-1)*n*a)/(1+(j-1)*b)

% Substitute in pxx_j and obtain
p00_j = simplify(subs(p00_j))
p01_j = simplify(subs(p01_j))
p10_j = simplify(subs(p10_j))
p11_j = simplify(subs(p11_j))
p12_j = simplify(subs(p12_j)) 

% gives:
% p00_j = (b*j+1)*a/(b*j+1+a*n*j)
% p01_j = a/(b*j+1+a*n*j)
% p11_j = (b^2*j+b+a*n*b*j+a)/(b*j+1+a*n*j)/(b*j+1)
% p12_j = a/(b*j+1+a*n*j)/(b*j+1)

% And the result is obtained






